package com.care.bigscreen.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.care.bigscreen.entity.StStation;
import com.care.bigscreen.vo.*;
import com.care.common.util.PageUtil;
import org.apache.ibatis.annotations.Param;

import java.util.List;


/**
 * @author stw
 * @version 1.0
 * @description 站点统计表DAO层接口
 * @date 2021/05/20/19:41
 */
public interface BigSreenMapper extends BaseMapper<StStation> {

    /**
     * 运营商大屏统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    OpBigScreenStatisticsVO selectBigScreenStatisticsByOp(@Param("stationId") Long stationId);

    /**
     * 运营商大屏统计查询
     * @return 根据经纬度范围搜索符合条件的大屏统计数据
     */
    OpBigScreenStatisticsVO selectOpBigScreenStatisticsByLongitudeLatitude(@Param("longitudeLeftUp") String longitudeLeftUp, @Param("latitudeLeftUp") String latitudeLeftUp, @Param("longitudeRightDown") String longitudeRightDown, @Param("latitudeRightDown")String latitudeRightDown);



    /**
     * 机构大屏统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    OrgBigScreenStatisticsVO selectBigScreenStatisticsByOrg(@Param("stationId") Long stationId, @Param("orgId") Long orgId);


    /**
     * 大屏统计查询（机构汇总表查询）
     * @return 搜索符合条件的大屏统计数据
     */
    BigScreenOrgVO selectBigScreenOrgStatistics(@Param("orgId") Long orgId);

    /**
     * 大屏统计查询
     * @return 根据经纬度范围搜索符合条件的大屏统计数据
     */
    OrgBigScreenStatisticsVO selectOrgBigScreenStatisticsByLongitudeLatitude(@Param("longitudeLeftUp") String longitudeLeftUp, @Param("latitudeLeftUp") String latitudeLeftUp, @Param("longitudeRightDown") String longitudeRightDown, @Param("latitudeRightDown")String latitudeRightDown, @Param("orgId") Long orgId);

    /**
     * 站点列表查询
     */
    List<BigScreenStationVO> selectBigScreenStationList(@Param("orgId") Long orgId);

    /**
     * 服务站大屏的站点统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    StationBigScreenStatisticsVO selectStationBigScreenStatisticsByStation(@Param("stationId") Long stationId);

    /**
     * 告警用户列表查询
     */
    List<BigScreenAlarmOlderVO> selectBigScreenAlarmOlderList(@Param("stationId") Long stationId,@Param("olderName") String olderName);

    /**
     * 服务站大屏的被监控人详细查询
     */
    BigScreenOlderDetailVO selectStationBigScreenOlderDetail(@Param("olderId") Long olderId);

    /**
     * 服务站大屏的事件总计查询
     */
    BigScreenEventTotalVO selectStationBigScreenEventTotal(@Param("stationId") Long stationId);

    /**
     * 机构大屏或服务站大屏的实时安全事件列表查询
     */
    List<BigScreenEventVO> selectBigScreenRtEventList(@Param("orgId") Long orgId, @Param("stationId") Long stationId);

    /**
     * 机构大屏的设备列表查询
     */
    List<BigScreenDeviceVO> selectOrgBigScreenDeviceList(@Param("orgId") Long orgId, @Param("stationId") Long stationId, @Param("status") String status);

    /**
     * 机构大屏的年龄性别列表查询
     */
    List<BigScreenAgeSexVO> selectOrgBigScreenAgeSexList(@Param("orgId") Long orgId);

    /**
     * 机构大屏的居住类型比例列表查询
     */
    List<BigScreenLiveTypeVO> selectOrgBigScreenLiveTypeRateList(@Param("orgId") Long orgId);

    /**
     * 机构大屏的事件趋势列表查询
     */
    List<BigScreenEnventTrendVO> selectOrgBigScreenEventTrendList(@Param("orgId") Long orgId);

    /**
     * 机构大屏的事件数量查询
     */
    BigScreenEnventCountVO selectOrgBigScreenEventCount(@Param("orgId") Long orgId);

    /**
     * 服务站大屏的已完成安全事件总记录数
     */
    Integer selectBigScreenHisEventListByCount(@Param("stationId") Long stationId);
    /**
     * 服务站大屏的已完成安全事件列表分页查询
     */
    List<BigScreenEventVO> selectBigScreenHisEventListByPage(@Param("stationId") Long stationId, @Param("pageUtil") PageUtil<BigScreenEventVO> pageUtil);


    /**
     * 服务站大屏的告警用户卡片列表总记录数
     */
    BigScreenOlderCardTotalVO selectStationBigScreenOlderCardTotal(@Param("stationId") Long stationId,@Param("olderName") String olderName);

    /**
     * 服务站大屏的用户卡片列表总记录数
     */
    Integer selectBigScreenOlderCardListByCount(@Param("stationId") Long stationId, @Param("olderName") String olderName,@Param("isAlarm") String isAlarm);

    /**
     * 服务站大屏的用户卡片列表分页查询
     */
    List<BigScreenOlderDetailVO> selectBigScreenOlderCardListByPage(@Param("stationId") Long stationId,@Param("olderName") String olderName,@Param("isAlarm") String isAlarm, @Param("pageUtil") PageUtil<BigScreenOlderDetailVO> pageUtil);


}
