package com.care.bms.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.bms.service.BmsMemberInfoService;
import com.care.common.annotation.Permission;
import com.care.common.exception.BDException;
import com.care.common.util.PageResult;
import com.care.common.util.Result;
import com.care.common.util.WebPageUtils;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.memberinfo.MemberInfoVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: stw
 * @Date: 2021/8/26
 * @Desc:
 */
@RestController
@Api(value = "MemberInfoController", tags = { "后台用户管理" })
@Slf4j
@RequestMapping("/bms/memberinfo")
@Permission
public class MemberInfoController {

    @Autowired
    private BmsMemberInfoService bmsMemberInfoService;


    @GetMapping("/listMemberInfo")
    @ApiOperation(value = "用户列表分页--二期新增")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "name", value = "用户名"),
            @ApiImplicitParam(paramType = "query", name = "phone", value = "电话"),
    })
    public PageResult<List<MemberInfoVO>> listMemberInfo(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                         @RequestParam(value = "name", required = false) String name,
                                                         @RequestParam(value = "phone", required = false) String phone,
                                                         PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<MemberInfoVO> pageResponse = this.bmsMemberInfoService.listCareMemberInfo(loginUser, name,phone,pageReqVo);
            if(pageResponse != null){
                return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
            } else {
                return null;
            }

        }catch (BDException e) {
            log.error("用户列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("用户管理: 用户列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }


//
//    @PostMapping("/addMemberInfo")
//    @ApiOperation(value = "新增用户")
//    public Result<Object> addMemberInfo(@RequestHeader("token") String token,
//                                   @RequestBody MemberInfoVO vo){
//        try {
//            this.bmsMemberInfoService.createCareMemberInfo(vo);
//            return  Result.success("新增成功！");
//        }catch (BDException e) {
//            log.error("新增用户-出现异常",e);
//            return Result.error(e.getMessage());
//        } catch (Exception e) {
//            log.error("用户管理: 新增用户出现异常",e);
//            return Result.error("新增失败！");
//        }
//    }


    @PostMapping("/updateMemberInfoStatus/{memberId}/{status}")
    @ApiOperation(value = "修改用户状态，状态：0可用，1停用   --二期新增")
    public Result<Object> updateMemberInfoStatus(@RequestHeader("token") String token,
                                                 @PathVariable("memberId") Long memberId,
                                                 @PathVariable("status") String status) {
        if (this.bmsMemberInfoService.updateCareMemberInfoStatus(memberId, status)) {
            return Result.success("更新成功！");
        } else {
            return Result.error("更新失败！");
        }
    }

//    @PostMapping("/updateMemberInfo")
//    @ApiOperation(value = "修改用户")
//    public Result<Object> updateMemberInfo(@RequestHeader("token") String token,
//                                 @RequestBody MemberInfoVO vo){
//        try {
//            this.bmsMemberInfoService.updateCareMemberInfo(vo);
//            return  Result.success("修改成功！");
//        }catch (BDException e) {
//            log.error("修改用户-出现异常",e);
//            return Result.error(e.getMessage());
//        } catch (Exception e) {
//            log.error("用户管理: 修改用户出现异常",e);
//            return Result.error("修改失败！");
//        }
//    }

//    @GetMapping("/getMemberInfoInfo/{memberId}")
//    @ApiOperation(value = "用户详情")
//    public Result<MemberInfoVO> getMemberInfoInfo(@RequestHeader("token") String token,@PathVariable("memberId") Long memberId){
//        MemberInfoVO orderInfo = this.bmsMemberInfoService.getCareMemberInfoById(memberId);
//        return  Result.success("查询成功！",orderInfo);
//    }
//
//    @PostMapping("/deleteMemberInfo/{memberId}")
//    @ApiOperation(value = "删除用户")
//    public Result<Object> deleteMemberInfo(HttpServletRequest request,@RequestHeader("token") String token,
//                                 @PathVariable("memberId") Long memberId){
//        try {
//            this.bmsMemberInfoService.deleteById(memberId);
//            return  Result.success("删除成功！");
//        } catch (Exception e) {
//            log.error("用户管理: 删除出现异常",e);
//            return Result.error("删除失败！");
//        }
//    }
}
