package com.care.bms.controller;


import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.bms.service.OrganizationService;
import com.care.common.annotation.Permission;
import com.care.common.entity.CareSysUser;
import com.care.common.exception.BDException;
import com.care.common.service.CareSysUserService;
import com.care.common.util.PageResult;
import com.care.common.util.Result;
import com.care.common.util.WebPageUtils;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.org.OrganizationVO;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "OrganizationController", tags = { "机构管理" })
@Slf4j
@RequestMapping("/bms/org")
@Permission
public class OrganizationController {

    @Autowired
    private OrganizationService organizationService;

    @Resource
    private CareSysUserService careSysUserService;
    /**
     *服务站下拉列表
     * @param request
     * @return
     */
    @GetMapping("/list4Select")
    @ApiOperation(tags = {"机构管理","设备管理"},value = "机构下拉列表 ----四期新增")
    public Result<List<OrganizationVO>> list4Select(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            List<OrganizationVO> datas = this.organizationService.queryOrgsForSelect();
            return Result.success(datas);
        }catch (Exception e) {
            log.error("机构管理: 机构下拉列表查询出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

    @GetMapping("/listOrganization")
    @ApiOperation(value = "机构列表分页  --四期新增")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "name", value = "机构名称"),
            @ApiImplicitParam(paramType = "systemName", name = "systemName", value = "系统名称")
    })
    public PageResult<List<OrganizationVO>> listOrganization(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                             @RequestParam(value = "name", required = false) String name,
                                                             @RequestParam(value = "systemName", required = false) String systemName,
                                                             PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<OrganizationVO> pageResponse = this.organizationService.listOrganization(name,systemName,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("机构列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("机构管理: 机构列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }

    @GetMapping("/synOrganization")
    @ApiOperation(value = "同步机构 -- 四期新增")
    public Result<Object> synOrganization(@RequestHeader("token") String token) {
        try {
            if (organizationService.synOrganization()){
                return  Result.success("同步成功！");
            } else {
                return Result.error("同步失败！");
            }

        }catch (BDException e) {
            log.error("同步机构-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("同步机构: 同步机构出现异常",e);
            return Result.error("同步失败！");
        }
    }


    @ApiOperation(value = "修改机构 -logo图片上传 -- 四期新增")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "放在请求头中的令牌",
                    dataType = "String", paramType = "header",
                    required = true)
    })
    @CrossOrigin
    @PostMapping(value = "/img/upload", headers="content-type=multipart/form-data",produces = "application/json;charset=UTF-8")
    public Result<String> imgUpload( @ApiParam(value="图片",required=true) MultipartFile file,
                                     @RequestHeader(name = "token") String token) {
        try {
            String url = organizationService.upload(file);
            return  Result.success("上传成功！",url);
        } catch (BDException e) {
            log.error("logo图片上传出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("机构管理: logo图片上传出现异常",e);
            return Result.error("logo图片上传失败！");
        }
    }

    @PostMapping("/updateOrganization")
    @ApiOperation(value = "修改机构 --四期新增")
    public Result<Object> updateOrganization(@RequestHeader("token") String token,
                                        @RequestBody OrganizationVO vo){
        try {
            organizationService.updateOrganization(vo);
            return Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改机构-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("机构管理: 修改机构出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @PostMapping("/invalidOrganization")
    @ApiOperation(value = "启用停用机构（只设置id和validStatus（0启用，1停用）） --四期新增")
    public Result<Object> invalidOrganization(@RequestHeader("token") String token,
                                             @RequestBody OrganizationVO vo){
        try {
            organizationService.invalidOrganization(vo);
            return Result.success("操作成功！");
        }catch (BDException e) {
            log.error("启用停用机构-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("机构管理: 启用停用机构出现异常",e);
            return Result.error("启用停用机构失败！");
        }
    }

    @GetMapping("/getOrganizationInfo/{organizationId}")
    @ApiOperation(value = "机构详情  --四期新增")
    public Result<OrganizationVO> getOrganizationInfo(@RequestHeader("token") String token,@PathVariable("organizationId") Long organizationId){
        OrganizationVO vo = this.organizationService.getOrganizationInfo(organizationId);
        if(vo != null){
            return  Result.success("查询成功！",vo);
        } else {
            return  Result.error("查询失败！");
        }

    }

    @PostMapping("/resetOrgAccountPassword/{orgAccountId}")
    @ApiOperation(value = "重置密码 --四期新增")
    public Result<Object> resetOrgAccountPassword(@RequestHeader("token") String token,
                                                      @PathVariable("orgAccountId") Long orgAccountId){
        try {
            UpdateWrapper<CareSysUser> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(CareSysUser::getId,orgAccountId)
                    .set(CareSysUser::getPassword, DigestUtils.md5Hex("123456"))
                    .set(CareSysUser::getModifyTime, new Date());
            if (careSysUserService.update(updateWrapper)){
                return  Result.success("重置成功！");
            } else {
                return  Result.error("重置失败！");
            }

        }catch (BDException e) {
            log.error("重置密码-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("机构管理: 重置密码出现异常",e);
            return Result.error("重置密码失败！");
        }
    }


}
