package com.care.bms.controller;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.bms.service.BmsSysUserService;
import com.care.common.annotation.Permission;
import com.care.common.entity.CareSysUser;
import com.care.common.exception.BDException;
import com.care.common.service.CareSysUserService;
import com.care.common.util.PageResult;
import com.care.common.util.Result;
import com.care.common.util.WebPageUtils;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.sysuser.SysUserVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "StationAccountController", tags = { "服务站账号管理" })
@Slf4j
@RequestMapping("/bms/sysuser")
@Permission
public class StationAccountController {

    @Autowired
    private BmsSysUserService bmsSysUserService;

    @Autowired
    private CareSysUserService careSysUserService;

    @GetMapping("/listStationAccount")
    @ApiOperation(value = "服务站账号列表分页  --四期新增")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "stationName", value = "服务站名称")
    })
    public PageResult<List<SysUserVO>> listStationAccount(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                          @RequestParam(value = "stationName", required = false) String stationName,
                                                          PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<SysUserVO> pageResponse = this.bmsSysUserService.listStationAccount(stationName,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("服务站账号列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站账号管理: 服务站账号列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }


    @PostMapping("/addStationAccount")
    @ApiOperation(value = "新增服务站账号 --四期新增")
    public Result<Object> addStationAccount(HttpServletRequest request,@RequestHeader("token") String token,
                                   @RequestBody SysUserVO vo){
        try {

            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            int flag = this.bmsSysUserService.createStationAccount(vo,loginUser);
            if (flag == 1) {
                return  Result.error("电话已存在！");
            } else if (flag == 0) {
                return  Result.success("新增成功！");
            } else {
                return  Result.error("新增失败！");
            }

        }catch (BDException e) {
            log.error("新增服务站账号-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站账号管理: 新增服务站账号出现异常",e);
            return Result.error("新增失败！");
        }
    }

    @PostMapping("/updateStationAccount")
    @ApiOperation(value = "修改服务站账号 （启用停用也用这个接口，只设置id和validStatus（0启用，1停用）） --四期新增")
    public Result<Object> updateStationAccount(HttpServletRequest request,@RequestHeader("token") String token,
                                 @RequestBody SysUserVO vo){
        try {
            this.bmsSysUserService.updateStationAccount(vo);
            return  Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改服务站账号-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站账号管理: 修改服务站账号出现异常",e);
            return Result.error("修改失败！");
        }
    }


    @PostMapping("/resetStationAccountPassword/{stationAccountId}")
    @ApiOperation(value = "重置密码 --四期新增")
    public Result<Object> resetStationAccountPassword(@RequestHeader("token") String token,
            @PathVariable("stationAccountId") Long stationAccountId){
        try {
            UpdateWrapper<CareSysUser> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(CareSysUser::getId,stationAccountId)
                    .set(CareSysUser::getPassword, DigestUtils.md5Hex("123456"))
                    .set(CareSysUser::getModifyTime, new Date());
            if (careSysUserService.update(updateWrapper)){
                return  Result.success("重置成功！");
            } else {
                return  Result.error("重置失败！");
            }

        }catch (BDException e) {
            log.error("重置密码-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站账号管理: 重置密码出现异常",e);
            return Result.error("重置密码失败！");
        }
    }

    @GetMapping("/getStationAccountInfo/{stationAccountId}")
    @ApiOperation(value = "服务站账号详情  --四期新增")
    public Result<SysUserVO> getStationAccountInfo(@RequestHeader("token") String token, @PathVariable("stationAccountId") Long stationAccountId){
        SysUserVO orderInfo = this.bmsSysUserService.getStationAccountById(stationAccountId);
        return  Result.success("查询成功！",orderInfo);
    }

}
