package com.care.bms.controller;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.bms.service.DeviceService;
import com.care.bms.service.StationService;
import com.care.common.annotation.Permission;
import com.care.common.entity.CareDevice;
import com.care.common.entity.CareStation;
import com.care.common.exception.BDException;
import com.care.common.service.CareDeviceService;
import com.care.common.service.CareStationService;
import com.care.common.util.PageResult;
import com.care.common.util.Result;
import com.care.common.util.WebPageUtils;

import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.device.DeviceVO;
import com.care.common.vo.station.StationVO;
import com.care.common.vo.station.StationVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "StationController", tags = { "服务站管理" })
@Slf4j
@RequestMapping("/bms/station")
@Permission
public class StationController {

    @Autowired
    private StationService stationService;

    @Resource
    private CareStationService careStationService;

    @Autowired
    private DeviceService deviceService;
    @Autowired
    private CareDeviceService careDeviceService;


    /**
     *服务站下拉列表
     * @param request
     * @return
     */
    @GetMapping("/list4Select")
    @ApiOperation(tags = {"房屋管理","设备管理"},value = "服务站下拉列表")
    public Result<List<StationVO>> list4Select(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<StationVO> datas = this.stationService.queryStationsForSelect(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("服务站管理: 服务站下拉列表查询出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

    @PostMapping("/addStation")
    @ApiOperation(value = "新增服务站 -- 四期新增")
    public Result<Object> addStation(HttpServletRequest request,@RequestHeader("token") String token,
                                     @RequestBody StationVO vo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            vo.setOrgId(loginUser.getOrgId());
            stationService.addStation(vo);
            return  Result.success("新增成功！");
        }catch (BDException e) {
            log.error("新增服务站-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("新增服务站: 新增服务站出现异常",e);
            return Result.error("新增失败！");
        }
    }



    @PostMapping("/updateStation")
    @ApiOperation(value = "修改服务站 --四期新增")
    public Result<Object> updateStation(@RequestHeader("token") String token,
                                        @RequestBody StationVO vo){
        try {
            CareStation careStation = new CareStation();
            BeanUtil.copyProperties(vo,careStation);
            this.careStationService.updateById(careStation);
            return  Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改服务站-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站管理: 修改服务站出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @GetMapping("/getStationInfo/{stationId}")
    @ApiOperation(value = "服务站详情  --四期新增")
    public Result<StationVO> getStationInfo(@RequestHeader("token") String token,@PathVariable("stationId") Long stationId){
        StationVO vo = this.stationService.getStationInfo(stationId);
        return  Result.success("查询成功！",vo);
    }

    @GetMapping("/listStation")
    @ApiOperation(value = "服务站列表分页  --四期新增")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "name", value = "服务站名称")
    })
    public PageResult<List<StationVO>> listStation(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                       @RequestParam(value = "name", required = false) String name,
                                                       PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<StationVO> pageResponse = this.stationService.listStation(name,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("服务站列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站管理: 服务站列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }

    @GetMapping("/listDeviceNonStation")
    @ApiOperation(value = "查询站点为空的设备列表分页--四期期新增")
    public PageResult<List<DeviceVO>> listDeviceNonStation(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                           PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<DeviceVO> pageResponse = this.deviceService.listDeviceNonStation(loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("设备列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("设备管理: 设备列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }

    @PostMapping("/distributeDevice")
    @ApiOperation(value = "分配设备(对象list中只设置stationId和设备的id) --四期新增 ")
    public Result<Object> distributeDevice( @RequestHeader("token") String token,
                                            @RequestBody List<DeviceVO> vos) {
        try {

            if (CollUtil.isEmpty(vos)) {
                return Result.error("分配设备列表不能为空！");
            }
            List<CareDevice> careDevices = new ArrayList<>();
            for (DeviceVO vo : vos) {
                CareDevice careDevice = new CareDevice();
                BeanUtil.copyProperties(vo, careDevice);
                careDevices.add(careDevice);
            }

            this.careDeviceService.updateBatchById(careDevices);

            return Result.success("分配设备成功！");
        } catch (BDException e) {
            log.error("分配设备-出现异常", e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("设备管理: 分配设备出现异常", e);
            return Result.error("分配设备失败！");
        }
    }
}
