package com.care.bms.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.bms.service.BmsSysUserService;
import com.care.common.annotation.Permission;
import com.care.common.exception.BDException;
import com.care.common.util.PageResult;
import com.care.common.util.Result;
import com.care.common.util.WebPageUtils;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.device.DeviceVO;
import com.care.common.vo.sysuser.SysUserVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "SysUserController", tags = { "后台管家管理" })
@Slf4j
@RequestMapping("/bms/sysuser")
@Permission
public class SysUserController {

    @Autowired
    private BmsSysUserService bmsSysUserService;

    /**
     *管家下拉列表
     * @param request
     * @return
     */
    @GetMapping("/listKeeper4Select")
    @ApiOperation(value = "登陆人所属机构、服务站下的管家下拉列表")
    public Result<List<SysUserVO>> listNotHandle(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                 @RequestParam(name = "stationId", required = false) Long stationId){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            Long orgId = loginUser.getOrgId();
            if (stationId==null){
                stationId = loginUser.getStationId();
            }

            List<SysUserVO> datas = this.bmsSysUserService.queryKeeperListByStationId(orgId,stationId);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("某个服务站下的管家下拉列表出现异常",e);
            return Result.error( "获取列表失败");
        }
    }


    @GetMapping("/listKeeper")
    @ApiOperation(value = "管家列表分页  --二期新增")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "name", value = "管家姓名"),
            @ApiImplicitParam(paramType = "query", name = "phone", value = "电话"),
            @ApiImplicitParam(paramType = "query", name = "stationId", value = "服务站ID"),
    })
    public PageResult<List<SysUserVO>> listKeeper(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                  @RequestParam(value = "name", required = false) String name,
                                                  @RequestParam(value = "phone", required = false) String phone,
                                                  @RequestParam(value = "stationId", required = false) Long stationId,
                                                  PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<SysUserVO> pageResponse = this.bmsSysUserService.listCareSysUser(name,phone,stationId,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("管家列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("管家管理: 管家列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }

    @GetMapping("/getKeeperFreeDev")
    @ApiOperation(value = "获取没分配给管家的空闲设备列表 --二期新增")
    public Result<List<DeviceVO>> getKeeperFreeDev(HttpServletRequest request, @RequestHeader("token") String token){
        UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
        List<DeviceVO> list = this.bmsSysUserService.getFreeDeviceVOList(loginUser);
        return  Result.success("查询成功！",list);
    }

    @PostMapping("/addKeeper")
    @ApiOperation(value = "新增管家 --二期新增")
    public Result<Object> addKeeper(HttpServletRequest request,@RequestHeader("token") String token,
                                   @RequestBody SysUserVO vo){
        try {

            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            String md5Password = DigestUtils.md5Hex("123456");
            vo.setPassword(md5Password);
            int flag = this.bmsSysUserService.createCareSysUser(vo,loginUser);
            if (flag == 1) {
                return  Result.error("电话已存在！");
            } else if (flag == 0) {
                return  Result.success("新增成功！");
            } else {
                return  Result.error("新增失败！");
            }

        }catch (BDException e) {
            log.error("新增管家-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("管家管理: 新增管家出现异常",e);
            return Result.error("新增失败！");
        }
    }

    @PostMapping("/updateKeeper")
    @ApiOperation(value = "修改管家 --二期新增")
    public Result<Object> updateKeeper(HttpServletRequest request,@RequestHeader("token") String token,
                                 @RequestBody SysUserVO vo){
        try {
            this.bmsSysUserService.updateCareSysUser(vo);
            return  Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改管家-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("管家管理: 修改管家出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @GetMapping("/getKeeperInfo/{chambId}")
    @ApiOperation(value = "管家详情  --二期新增")
    public Result<SysUserVO> getKeeperInfo(HttpServletRequest request, @RequestHeader("token") String token, @PathVariable("chambId") Long chambId){
        UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
        SysUserVO orderInfo = this.bmsSysUserService.getCareSysUserById(chambId,loginUser);
        return  Result.success("查询成功！",orderInfo);
    }

    @PostMapping("/deleteKeeper/{chambId}")
    @ApiOperation(value = "删除管家")
    public Result<Object> deleteKeeper(HttpServletRequest request,@RequestHeader("token") String token,
                                 @PathVariable("chambId") Long chambId){
        try {
            if( this.bmsSysUserService.deleteById(chambId)){
                return  Result.success("删除成功！");
            } else {
                return Result.error("管家被引用，删除失败！");
            }

        } catch (Exception e) {
            log.error("管家管理: 删除出现异常",e);
            return Result.error("删除失败！");
        }
    }
}
