package com.care.client.controller;

import cn.hutool.core.util.XmlUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.care.client.dto.TemplateDataDTO;
import com.care.client.service.PinanbaoContactService;
import com.care.client.vo.HouseContactVO;
import com.care.client.vo.WxAccessTokenVO;
import com.care.common.cache.RedisKeyConstant;
import com.care.common.cache.RedisUtil;
import com.care.common.entity.CareDevice;
import com.care.common.entity.CareHouseContact;
import com.care.common.service.CareDeviceService;
import com.care.common.service.CareHouseContactService;
import com.care.common.service.WxQrCodeService;
import com.care.common.service.WxTemplateService;
import com.care.common.util.CheckoutUtil;
import com.care.common.util.MessageUtil;
import com.care.common.util.WxTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@Controller
@RequestMapping("/wxmsg")
@Slf4j
public class WxMsgController {

//    @Resource
//    private CareOfficialMemberInfoService careOfficialMemberInfoService;

//    @Resource
//    private CareMemberInfoService careMemberInfoService;

    @Resource
    private WxTemplateService wxService;

    @Resource
    private PinanbaoContactService pinanbaoContactService;

    @Resource
    private WxTemplateService wxTemplateService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private CareDeviceService careDeviceService;

    @Resource
    private WxQrCodeService wxQrCodeService;

    /**
     * 微信消息接收和token验证
     * @param request
     * @param response
     * @throws IOException
     */
        @RequestMapping(value="/checkToken",method= RequestMethod.GET)
    public void weChat(HttpServletRequest request, HttpServletResponse response) throws IOException {
        log.info("校验wxToken 开始");
        boolean isGet = request.getMethod().toLowerCase().equals("get");
        if (isGet) {
            // 微信加密签名
            String signature = request.getParameter("signature");
            // 时间戳
            String timestamp = request.getParameter("timestamp");
            // 随机数
            String nonce = request.getParameter("nonce");
            // 随机字符串
            String echostr = request.getParameter("echostr");
            log.info("校验wxToken signature: " + signature);
            // 通过检验signature对请求进行校验，若校验成功则原样返回echostr，表示接入成功，否则接入失败
            if (signature != null && CheckoutUtil.checkSignature(signature, timestamp, nonce)) {
                try {
                    boolean flag = CheckoutUtil.checkSignature(signature, timestamp, nonce);
                    System.out.println(flag);
                    PrintWriter print = response.getWriter();
                    print.write(echostr);
                    System.out.println(echostr);
                    print.flush();
                    print.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @RequestMapping(value="/checkToken",method=RequestMethod.POST)
    @ResponseBody
    public String  responseEvent(HttpServletRequest req, HttpServletResponse resp)throws IOException {
        req.setCharacterEncoding("UTF-8");
        resp.setCharacterEncoding("UTF-8");
        String message = "success";
        try {
//            WxAccessTokenVO wxAccessTokenVO = wxService.processAccessToken();
            WxAccessTokenVO wxAccessTokenVO = wxQrCodeService.processOfficialToken();
            if (wxAccessTokenVO == null) {
                return "failure";
            }
            // 读取输入流
            Document document = XmlUtil.readXML(req.getInputStream());
            // 得到xml根元素
            Element element = document.getDocumentElement();
            //把微信返回的xml信息转义成map
            Map<String, Object> map = XmlUtil.xmlToMap(element);
            System.out.println("微信接收到的消息为:"+map.toString());
            String fromUserName = ObjectUtils.toString(map.get("FromUserName"));//消息来源用户标识
            String msgType = ObjectUtils.toString(map.get("MsgType"));//消息类型(event或者text)
            String eventType = ObjectUtils.toString(map.get("Event"));//事件类型
//            String nickName = getUserNickName(fromUserName);
            if(MessageUtil.MSGTYPE_EVENT.equals(msgType)){//如果为事件类型
                if(MessageUtil.SCAN_EVENT.equalsIgnoreCase(eventType)) {// 扫描事件
                    JSONObject parseUnionObject = WxTemplateUtil.parseWxOpenIdBySubscribe(wxAccessTokenVO.getAccessToken(), fromUserName);
                    System.out.println("scan parseUnionObject:" + parseUnionObject.toString());

                    String eventKey = ObjectUtils.toString(map.get("EventKey"));
                    Object obj = redisUtil.get(RedisKeyConstant.YUANSERVICE_QRCODE + ":" + eventKey);
                    log.info("用户扫描: " + fromUserName + ": " + ObjectUtils.toString(map.get("EventKey")));
                    if (obj == null) {
                        return "failure";
                    }
                    String[] arrs = Objects.toString(obj).split(":");
                    if (arrs == null || arrs.length != 2) {
                        return "failure";
                    }
                    CareDevice careDevice = careDeviceService.getById(arrs[0]);
                    if (careDevice == null) {
                        log.info("设备不存在: " + arrs[0]);
                        return "failure";
                    }
                    if (parseUnionObject != null) {
                        HouseContactVO houseContactVO = new HouseContactVO();
                        houseContactVO.setContactName(parseUnionObject.getStr("nickname"));
                        houseContactVO.setCreateTime(new Date());
                        String gender = "";
                        if ("0".equalsIgnoreCase(parseUnionObject.getStr("sex"))) {
                            gender = "M";
                        } else if ("1".equalsIgnoreCase(parseUnionObject.getStr("sex"))) {
                            gender = "W";
                        }
                        houseContactVO.setGender(gender);
                        houseContactVO.setOpenId(fromUserName);
                        houseContactVO.setMemberId(NumberUtils.toLong(arrs[1]));
                        boolean saved = pinanbaoContactService.createOrUpdateMyContact(NumberUtils.toLong(arrs[0]), houseContactVO);
                        if (!saved) {
                            log.error("保存订阅用户出现错误: " + JSONUtil.toJsonStr(houseContactVO));
                            return "failure";
                        }

                        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        // 正式环境
                        String templateId = "W7tT8QGmrBQNVimt6l6w55DBwSH5MBUiFgJxf3gjIV4";
                        // 测试环境
//                        String templateId = "gvJ1g3JOThT5IuQc1In6D2cwKwmKz9MBIkGdkXtXOTI";

                        Map<String, TemplateDataDTO> data = new HashMap<>();
                        data.put("first", new TemplateDataDTO("您已关联该监护人", "#459ae9"));
                        data.put("keyword1", new TemplateDataDTO(careDevice.getDevName(), "#459ae9"));
                        data.put("keyword2", new TemplateDataDTO(simpleDateFormat.format(new Date()), "#459ae9"));
                        data.put("remark", new TemplateDataDTO("请补全您的联系信息, 便于能及时收到事件通知", "#459ae9"));

                        String openId = fromUserName;
                        log.info("生成小程序跳转URL: " + "pages/supplement/index?memberId=" + arrs[1] + "&openId=" + fromUserName);
                        boolean flag = wxTemplateService.sendTemplateMsgOriginal(openId, templateId, "wxda02870032b8f928", "pages/supplement/index?memberId=" + arrs[1] + "&openId=" + openId, data);
                        log.info("发送消息模板: openId: " + openId + ", templateId: " + templateId + " 状态: " + flag);

                        return "success";
                    }

                } else if (MessageUtil.SUBSCIBE_EVENT.equals(eventType)) {//处理订阅事件
                    JSONObject parseUnionObject = WxTemplateUtil.parseWxOpenIdBySubscribe(wxAccessTokenVO.getAccessToken(), fromUserName);
                    System.out.println("subscibe parseUnionObject:" + parseUnionObject.toString());
//                    parseUnionObject.set("unionid", "oyVYT6Ox9b88n_33lgKWRj5uFbUY");

                    String eventKey = ObjectUtils.toString(map.get("EventKey"));
                    if (StringUtils.isNotBlank(eventKey) && eventKey.indexOf("_") != -1) {
                        eventKey = StringUtils.substring(eventKey, eventKey.indexOf("_") + 1);
                    }
                    Object obj = redisUtil.get(RedisKeyConstant.YUANSERVICE_QRCODE + ":" + eventKey);
                    log.info("新用户关注: " + fromUserName + ": " + ObjectUtils.toString(map.get("EventKey")));
                    if (obj == null) {
                        return "failure";
                    }

                    String[] arrs = Objects.toString(obj).split(":");
                    if (arrs == null || arrs.length != 2) {
                        return "failure";
                    }
                    CareDevice careDevice = careDeviceService.getById(arrs[0]);
                    if (careDevice == null) {
                        log.info("设备不存在: " + arrs[0]);
                        return "failure";
                    }

                    if (parseUnionObject != null) {
//                        CareOfficialMemberInfo careOfficialMemberInfo = new CareOfficialMemberInfo();
//                        careOfficialMemberInfo.setNickname(parseUnionObject.getStr("nickname"));
//                        careOfficialMemberInfo.setOpenId(fromUserName);
//                        careOfficialMemberInfo.setProfile(parseUnionObject.getStr("headimgurl"));
//                        careOfficialMemberInfo.setUnionId(parseUnionObject.getStr("unionid"));
//                        careOfficialMemberInfo.setSubscribeStatus(1);
//                        careOfficialMemberInfo.setCreateTime(new Date());
//                        boolean subscribed = careOfficialMemberInfoService.subscribe(careOfficialMemberInfo);
//                        if (subscribed) {
//                            return "success";
//                        } else {
//                            return "failure";
//                        }
                        HouseContactVO houseContactVO = new HouseContactVO();
                        houseContactVO.setContactName(parseUnionObject.getStr("nickname"));
                        houseContactVO.setCreateTime(new Date());
                        String gender = "";
                        if ("0".equalsIgnoreCase(parseUnionObject.getStr("sex"))) {
                            gender = "M";
                        } else if ("1".equalsIgnoreCase(parseUnionObject.getStr("sex"))) {
                            gender = "W";
                        }
                        houseContactVO.setGender(gender);
                        houseContactVO.setOpenId(fromUserName);
                        houseContactVO.setMemberId(NumberUtils.toLong(arrs[1]));
                        boolean saved = pinanbaoContactService.createOrUpdateMyContact(NumberUtils.toLong(arrs[0]), houseContactVO);
                        if (!saved) {
                            log.error("保存订阅用户出现错误: " + JSONUtil.toJsonStr(houseContactVO));
                            return "failure";
                        }

                        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        // 正式环境
                        String templateId = "W7tT8QGmrBQNVimt6l6w55DBwSH5MBUiFgJxf3gjIV4";
                        // 测试环境
//                        String templateId = "gvJ1g3JOThT5IuQc1In6D2cwKwmKz9MBIkGdkXtXOTI";

                        Map<String, TemplateDataDTO> data = new HashMap<>();
                        data.put("first", new TemplateDataDTO("您已关联该监护人", "#459ae9"));
                        data.put("keyword1", new TemplateDataDTO(careDevice.getDevName(), "#459ae9"));
                        data.put("keyword2", new TemplateDataDTO(simpleDateFormat.format(new Date()), "#459ae9"));
                        data.put("remark", new TemplateDataDTO("请补全您的联系信息, 便于能及时收到事件通知", "#459ae9"));

                        String openId = fromUserName;
                        log.info("生成小程序跳转URL: " + "pages/supplement/index?memberId=" + arrs[1] + "&openId=" + fromUserName);
                        boolean flag = wxTemplateService.sendTemplateMsgOriginal(openId, templateId, "wxda02870032b8f928", "pages/supplement/index?memberId=" + arrs[1] + "&openId=" + openId, data);
                        log.info("发送消息模板: openId: " + openId + ", templateId: " + templateId + " 状态: " + flag);

//                        String eventKey = ObjectUtils.toString(map.get("EventKey"));
//                        if (StringUtils.isNotBlank(eventKey) && eventKey.indexOf("_") != -1) {
//                            eventKey = StringUtils.substring(eventKey, eventKey.indexOf("_") + 1);
//                        }
//                        Object obj = redisUtil.get(RedisKeyConstant.YUANSERVICE_QRCODE + ":" + eventKey);
//                        if (obj == null) {
//                            return "failure";
//                        }

//                        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//                        String templateId = "OPENTM418029089";
//
//                        Map<String, TemplateDataDTO> data = new HashMap<>();
//                        data.put("first", new TemplateDataDTO("您已成功关联该设备", "#ff5200"));
//                        data.put("keyword1", new TemplateDataDTO("跌倒报警", "#000000"));
//                        data.put("keyword2", new TemplateDataDTO(simpleDateFormat.format(new Date()), "#000000"));
//                        data.put("remark", new TemplateDataDTO("请尽快补全您的联系信息, 便于能及时通知您", "#ff5200"));
//
//                        String openId = parseUnionObject.getStr("openid");
//                        boolean flag = wxTemplateService.sendTemplateMsg(parseUnionObject.getStr("openid"), templateId, "wxda02870032b8f928", "pages/supplement/index", data);
//                        log.info("发送消息模板: openId: " + openId + ", templateId: " + templateId + " 状态: " + flag);

//                        careHouseContactService.save();
                        return "success";
                    }
                } else if (MessageUtil.UNSUBSCIBE_EVENT.equals(eventType)) {//处理取消订阅事件
                    System.out.println("事件类型为:"+eventType);
//                    JSONObject parseUnionObject = WxTemplateUtil.parseWxOpenIdBySubscribe(wxAccessTokenVO.getAccessToken(), fromUserName);
//                    if (parseUnionObject != null && parseUnionObject.containsKey("unionid")) {
//                        boolean updated = careOfficialMemberInfoService.cancelSubscribe(parseUnionObject.getStr("unionid"));
//                        if (updated) {
//                            return "success";
//                        } else {
//                            return "failure";
//                        }
//                    }
                    return "success";
//                    message = MessageUtil.unsubscribe(toUserName, fromUserName);
                }
            }else {
//                //微信消息分为事件推送消息和普通消息,非事件即为普通消息类型
//                switch (msgType) {
//                    case "text":{//文本消息
//                        String content = map.get("Content");//用户发的消息内容
//                        content = "您发的消息内容是:"+content+",如需帮助,请联系";
//                        message = MessageUtil.replyMsg(toUserName, fromUserName,content,"text");
//                        break;
//                    }
//                    case "image":{//图片消息
//                        String content = "您发的消息内容是图片,如需帮助,请联系";
//                        message = MessageUtil.replyMsg(toUserName, fromUserName,content,"text");
//                        break;
//                    }
//                    default:{//其他类型的普通消息
//                        break;
//                    }
//                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("关注微信公众号自动回复的消息内容为:" + message);
        return message;
    }

}
