package com.care.client.schedule;



import com.care.common.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;


/** 健康数据统计调度
 * @author stw
 * @version 2.7.0.0 创建于 2021/11/16
 **/
@Configuration
@EnableScheduling
@EnableAsync
public class HealthDataStaSchedule {

    @Autowired
    private CareBedroomInbedHourStaService careBedroomInbedHourStaService;

    @Autowired
    private CareToiletInoutHourStaService careToiletInoutHourStaService;

    @Autowired
    private CareWalkingDistanceHourStaService careWalkingDistanceHourStaService;

    @Autowired
    private CareDevEventDayStaService careDevEventDayStaService;


    @Autowired
    private CareHeartBreathHourStaService careHeartBreathHourStaService;

    @Autowired
    private CareHeartBreathDetailService careHeartBreathDetailService;

    /**
     * 每小时卧室在床统计
     * 1分钟调用一次
     */
    @Async
    @Scheduled(cron = "0 0/1 * * * ?")
    public void exeBedroomInbedHourSta() {
        careBedroomInbedHourStaService.exeBedroomInbedHourSta();
    }

    /**
     * 每小时进出卫生间统计
     * 1小时调用一次
     */
    @Async
    @Scheduled(cron = "0 0/1 * * * ?")
    public void exeToiletInoutHourSta() {
        careToiletInoutHourStaService.exeToiletInoutHourSta();
    }

    /**
     * 每小时行走距离统计
     * 1分钟调用一次
     */
    @Async
    @Scheduled(cron = "0 0/1 * * * ?")
    public void exeWalkingDistanceHourSta() {
        careWalkingDistanceHourStaService.exeWalkingDistanceHourSta();
    }

    /**
     * 每日安全事件统计
     * 1分钟调用一次
     */
    @Async
    @Scheduled(cron = "0 0/1 * * * ?")
    public void exeDevEventDaySta() {
        careDevEventDayStaService.exeDevEventDaySta();
    }

    /**
     * 每小时心率呼吸统计
     * 1分钟调用一次
     */
    @Async
    @Scheduled(cron = "0 0/1 * * * ?")
    public void exeHeartBreathHourSta() {
        careHeartBreathHourStaService.exeHeartBreathHourSta();
    }

    /**
     * 心率呼吸详细采样
     * 1 分钟调用一次
     */
    @Async
    @Scheduled(cron = "0 0/1 * * * ?")
    public void exeHeartBreathDetail() {
        careHeartBreathDetailService.exeHeartBreathDetail();
    }
}
