package com.care.client.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.math.MathUtil;
import cn.hutool.core.util.NumberUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.care.client.vo.*;
import com.care.common.entity.*;
import com.care.common.enums.*;
import com.care.common.service.*;
import com.care.common.vo.device.DeviceVO;
import com.care.common.vo.order.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author: lilt
 * @Date: 2021/6/7
 * @Desc:
 */
@Slf4j
@Service
public class PinanbaoService {
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareEventOrderService careEventOrderService;
    @Resource
    private CareEventOrderHandleHisService careEventOrderHandleHisService;
    @Resource
    private CareHouseService careHouseService;
    @Resource
    private CareEventOrderChambService careEventOrderChambService;
    @Resource
    private CareSysUserService careSysUserService;
    @Resource
    private CareEventOrderContactStatusService careEventOrderContactStatusService;
    @Resource
    private CareEventOrderOlderService careEventOrderOlderService;
    @Resource
    private CareBedroomInbedDetailService careBedroomInbedDetailService;
    @Resource
    private CareToiletInoutDetailService careToiletInoutDetailService;
    @Resource
    private CareWalkingDistanceDetailService careWalkingDistanceDetailService;
    @Resource
    private CareBedroomInbedHourStaService careBedroomInbedHourStaService;
    @Resource
    private CareToiletInoutHourStaService careToiletInoutHourStaService;
    @Resource
    private CareWalkingDistanceHourStaService careWalkingDistanceHourStaService;
    /**
     * 查询我的的设备列表
     * @param memberId
     * @return
     */
    public List<DeviceVO> devList(Long memberId){
        return this.careDeviceService.queryListByMemberId(memberId);
    }

    /**
     * 设备监测数据
     * @param devId
     * @return
     */
    public DeviceMonitorVO devMonitorData(Long devId){
        CareDevice device = this.careDeviceService.getById(devId);
        DeviceMonitorVO vo = new DeviceMonitorVO();
        if (device!=null){
            vo.setId(devId);
            //场景
            vo.setDevScene(device.getDevScene());
            vo.setDevSceneName(DeviceSceneEnum.getCodeToName(device.getDevScene()));

            //心率和呼吸
            vo.setBreathRate(device.getBreathRate());
            vo.setHeartRate(device.getHeartRate());

            //厕所场景下， 活动状态 只有 有人/无人
            if (DeviceSceneEnum.TOILET.getValue().equals(device.getDevScene())){
                vo.setMoveStatu(device.getMoveStatus());
                String desc = DeviceMoveStateEnum.NONE.getValue().equals(device.getMoveStatus())?"无人":"有人";
                vo.setMoveStatuName(desc);
            }else{
                //活动状态
                vo.setMoveStatu(device.getMoveStatus());
                vo.setMoveStatuName(DeviceMoveStateEnum.getCodeToName(device.getMoveStatus()));
            }


            //在床状态
            vo.setInbedStatus(device.getInbedStatus());
            vo.setInbedStatusName(DeviceInbedStateEnum.getCodeToName(device.getInbedStatus()));

            //卧室场景： 在床、无人冲突时， 显示成 静止状态
            if (DeviceSceneEnum.BEDROOM.getValue().equals(device.getDevScene())){
                if (DeviceMoveStateEnum.NONE.getValue().equals(device.getMoveStatus()) && DeviceInbedStateEnum.INBED.getValue().equals(device.getInbedStatus())){
                    vo.setMoveStatu(DeviceMoveStateEnum.STATIC.getValue());
                    vo.setMoveStatuName(DeviceMoveStateEnum.getCodeToName(DeviceMoveStateEnum.STATIC.getValue()));
                }
            }

            //睡眠时间
            DeviceMonitorVO vo1 = careBedroomInbedDetailService.selectSleepTime(devId);
            if(vo1 != null) {
                vo.setSleepTimeHour(vo1.getSleepTimeHour());
                vo.setSleepTimeMinute(vo1.getSleepTimeMinute());
            }

            //卫生间进出次数
            DeviceMonitorVO vo2 = careToiletInoutDetailService.selectToiletInoutFrequency(devId);
            if(vo2 != null) {
                vo.setToiletInoutFrequency(vo2.getToiletInoutFrequency());
                vo.setToiletStayDurationAvg(vo2.getToiletStayDurationAvg());
                vo.setToiletStayDurationMax(vo2.getToiletStayDurationMax());
            }

            //活动距离和时长
            DeviceMonitorVO vo3 = careWalkingDistanceDetailService.selectWalking(devId);
            if(vo3 != null) {
                vo.setActivityDistance(vo3.getActivityDistance());
                vo.setActivityTimeHour(vo3.getActivityTimeHour());
                vo.setActivityTimeMinute(vo3.getActivityTimeMinute());
            }
        }
        return vo;
    }

    /**
     * 在床时间chart
     * @param devId
     * @return
     */
    public List<CareBedroomInbedHourSta> getInbedTimeChartList(Long devId){
        String beginHour = DateUtil.format(new Date(),"yyyyMMdd") + "00";
        QueryWrapper<CareBedroomInbedHourSta> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareBedroomInbedHourSta::getDevId,devId).ge(CareBedroomInbedHourSta::getDataHour,beginHour).orderByAsc(CareBedroomInbedHourSta::getDataHour);
        List<CareBedroomInbedHourSta> list = this.careBedroomInbedHourStaService.list(queryWrapper);
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item -> {
                item.setDataHour(item.getDataHour().substring(8)+":00");
            });
        }
        return list;
    }

    /**
     * 卫生间进出统计chart
     * @param devId
     * @return
     */
    public List<CareToiletInoutHourSta> getToiletInoutChartList(Long devId){
        String beginHour = DateUtil.format(new Date(),"yyyyMMdd") + "00";
        QueryWrapper<CareToiletInoutHourSta> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareToiletInoutHourSta::getDevId,devId).ge(CareToiletInoutHourSta::getDataHour,beginHour).orderByAsc(CareToiletInoutHourSta::getDataHour);
        List<CareToiletInoutHourSta> list = this.careToiletInoutHourStaService.list(queryWrapper);
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item -> {
                item.setDataHour(item.getDataHour().substring(8)+":00");
            });
        }
        return list;
    }

    /**
     * 活动统计chart
     * @param devId
     * @return
     */
    public List<CareWalkingDistanceHourSta> getActivityChartList(Long devId){
        String beginHour = DateUtil.format(new Date(),"yyyyMMdd") + "00";
        QueryWrapper<CareWalkingDistanceHourSta> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareWalkingDistanceHourSta::getDevId,devId).ge(CareWalkingDistanceHourSta::getDataHour,beginHour).orderByAsc(CareWalkingDistanceHourSta::getDataHour);
        List<CareWalkingDistanceHourSta> list = this.careWalkingDistanceHourStaService.list(queryWrapper);
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item -> {
                item.setDataHour(item.getDataHour().substring(8)+":00");
                item.setWalkingDistance(Math.ceil(item.getWalkingDistance()));
            });
        }
        return list;
    }

    /**
     * 查询正在进行中的事件
     * @param memberId
     * @return
     */
    public IPage<EventOrderVO> getAllListEvent(Integer current, Long memberId) {
        IPage<CareEventOrder> page = new Page<>(current, 10);
        List<DeviceVO> deviceVOS = devList(memberId);
        List<Long> deviceIds = deviceVOS.stream().map(DeviceVO::getDevId).collect(Collectors.toList()).stream().distinct().collect(Collectors.toList());
        if (CollUtil.isNotEmpty(deviceIds)){
            QueryWrapper<CareEventOrder> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(CareEventOrder::getMemberId,memberId).in(CareEventOrder::getDevId, deviceIds).orderByDesc(CareEventOrder::getCreateTime);
            IPage<CareEventOrder> contentPage = this.careEventOrderService.page(page, queryWrapper);
            IPage<EventOrderVO> results = new Page<>(contentPage.getCurrent(), contentPage.getSize(), contentPage.getTotal());
            if (CollUtil.isNotEmpty(contentPage.getRecords())) {
                List<EventOrderVO> list = new ArrayList<>();
                contentPage.getRecords().forEach(item -> {
                    EventOrderVO resVO = new EventOrderVO();
                    BeanUtils.copyProperties(item, resVO);
                    resVO.setOrderTypeName(OrderTypeEnum.getCodeToName(item.getOrderType()));
                    resVO.setStatusName(OrderStatusEnum.getCodeToName(item.getStatus()));
                    list.add(resVO);
                });
                results.setRecords(list);
            }
            return results;
        }else{
            return new Page<>(1, 10, 0);
        }


    }

    /**
     * 查询正在进行中的事件
     * @param memberId
     * @return
     */
    public EventOrderVO getDoingEvent(Long memberId){
        EventOrderVO vo = null;
        QueryWrapper<CareEventOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrder::getMemberId,memberId).and(item->
            item.eq(CareEventOrder::getStatus, OrderStatusEnum.TODO.getValue()).or(i->
                i.eq(CareEventOrder::getStatus,OrderStatusEnum.DOING.getValue())
            )
        ).orderByAsc(CareEventOrder::getCreateTime);
        List<CareEventOrder> list = this.careEventOrderService.list(queryWrapper);
        if (CollUtil.isNotEmpty(list)){
            CareEventOrder order = list.get(0);
            vo = new EventOrderVO();
            BeanUtil.copyProperties(order,vo);
        }
        return vo;
    }

    /**
     * 获取事件详情
     * @return
     */
    public EventDetailVO getEventInfo(Long orderId) {
        EventDetailVO result = new EventDetailVO();

        //事件基本信息
        EventOrderVO eventOrderVO = this.getOrderInfo(orderId);
        result.setEventOrderVO(eventOrderVO);

        //主管家和改派管家列表
        List<ChamberlainVO>  chambList = this.queryChambList(orderId);
        result.setChambList(chambList);

        //老人列表
        List<OrderOlderVO>  olderList = this.queryOrderOlderList(orderId);
        result.setOlderList(olderList);

        //联系人列表
        List<OrderContactorVO> contactList = this.queryContactList(orderId);
        result.setContactList(contactList);

        //历史处理记录
        List<OrderHandleHisVO> orderHandleHisVOList = queryOrderHandleHis(orderId);
        result.setOrderHandleHisVOList(orderHandleHisVOList);
        return result;
    }

    /**
     * 查询事件基本信息
     * @param orderId
     * @return
     */
    private EventOrderVO getOrderInfo(Long orderId){
        EventOrderVO eventOrderVO = null;
        CareEventOrder order = this.careEventOrderService.getById(orderId);
        if (order!=null){
            eventOrderVO  = new EventOrderVO();
            BeanUtil.copyProperties(order,eventOrderVO);
            eventOrderVO.setStatusName(OrderStatusEnum.getCodeToName(eventOrderVO.getStatus()));
            eventOrderVO.setOrderTypeName(OrderTypeEnum.getCodeToName(eventOrderVO.getOrderType()));
            CareHouse house = this.careHouseService.getById(order.getHouseId());
            CareHouseVO houseVO = new CareHouseVO();
            BeanUtil.copyProperties(house,houseVO);
            //houseVO.setRelationTypeDesc("屋主");
            eventOrderVO.setHouse(houseVO);
        }
        return  eventOrderVO;
    }

    /**
     * 主管家改派管家列表查询
     * @param orderId
     * @return
     */
    private List<ChamberlainVO>  queryChambList(Long orderId){
        List<ChamberlainVO> chambList = new ArrayList<>();

        ChamberlainVO mainVO = null;
        ChamberlainVO assignVO = null;

        CareEventOrder order = this.careEventOrderService.getById(orderId);
        CareDevice dev = this.careDeviceService.getById(order.getDevId());
        if(dev != null){
            CareSysUser careSysUser = this.careSysUserService.getById(dev.getChambId());
            if (careSysUser != null) {
                mainVO = new ChamberlainVO();
                BeanUtil.copyProperties(careSysUser,mainVO);
                mainVO.setIsMain(TrueFalseEnum.TRUE.getValue());
                mainVO.setStatusDesc(ChambStatusEnum.getCodeToName(mainVO.getStatus()));
            }
        }
        CareEventOrderChamb careEventOrderChamb = this.careEventOrderChambService.getChambOrderByOrderId(orderId);
        if(careEventOrderChamb != null) {
            CareSysUser careSysUser = this.careSysUserService.getById(careEventOrderChamb.getChambId());
            if(careSysUser != null){
                assignVO = new ChamberlainVO();
                BeanUtil.copyProperties(careSysUser,assignVO);
                assignVO.setIsMain(TrueFalseEnum.FALSE.getValue());
                assignVO.setStatusDesc(ChambStatusEnum.getCodeToName(mainVO.getStatus()));
            }
        }

        if (mainVO != null && assignVO != null) {
            if(mainVO.getId().equals(assignVO.getId())){ //没有指派管家
                if(ChambStatusEnum.BUSY.getValue().equals(mainVO.getStatus())){ //主状态是忙碌时拆分子状态
                    if(ChambOrderStatusEnum.DOING.getValue().equals(careEventOrderChamb.getStatus())){
                        mainVO.setSubStatus(ChambSubStatusEnum.DOING.getValue());//处理中
                    } else {
                        mainVO.setSubStatus(ChambSubStatusEnum.BUSY.getValue());//忙碌
                    }
                } else {//主状态不是忙碌时设置子状态和主状态相同
                    mainVO.setSubStatus(mainVO.getStatus());
                }
                mainVO.setSubStatusDesc(ChambStatusEnum.getCodeToName(mainVO.getSubStatus()));
                chambList.add(mainVO);
            } else{ //指派了管家
                //设置主管家子状态和主状态相同：空闲、休假、忙碌
                mainVO.setSubStatus(mainVO.getStatus());

                //设置指派管家状态
                if(ChambStatusEnum.BUSY.getValue().equals(assignVO.getStatus())) { //指派管家的主状态是忙碌时拆分子状态
                    if(ChambOrderStatusEnum.DOING.getValue().equals(careEventOrderChamb.getStatus())){
                        assignVO.setSubStatus(ChambSubStatusEnum.DOING.getValue());//处理中
                    } else {
                        assignVO.setSubStatus(ChambSubStatusEnum.BUSY.getValue());//忙碌
                    }
                } else {//指派管家的主状态不是忙碌时设置子状态和主状态相同
                    assignVO.setSubStatus(assignVO.getStatus());
                }
                mainVO.setSubStatusDesc(ChambStatusEnum.getCodeToName(mainVO.getSubStatus()));
                chambList.add(mainVO);
                assignVO.setSubStatusDesc(ChambStatusEnum.getCodeToName(assignVO.getSubStatus()));
                chambList.add(assignVO);
            }
        } else if(mainVO == null && assignVO != null) {
            //设置指派管家状态
            if(ChambStatusEnum.BUSY.getValue().equals(assignVO.getStatus())) { //指派管家的主状态是忙碌时拆分子状态
                if(ChambOrderStatusEnum.DOING.getValue().equals(careEventOrderChamb.getStatus())){
                    assignVO.setSubStatus(ChambSubStatusEnum.DOING.getValue());//处理中
                } else {
                    assignVO.setSubStatus(ChambSubStatusEnum.BUSY.getValue());//忙碌
                }
            } else {//指派管家的主状态不是忙碌时设置子状态和主状态相同
                assignVO.setSubStatus(assignVO.getStatus());
            }

            assignVO.setSubStatusDesc(ChambStatusEnum.getCodeToName(assignVO.getSubStatus()));
            chambList.add(assignVO);
        }

        return chambList;
    }

    /**
     * 查询老人列表
     * @param orderId
     * @return
     */
    public List<OrderOlderVO>  queryOrderOlderList(Long orderId){
        List<OrderOlderVO> orderOlderVOList = Lists.newArrayList();
        QueryWrapper<CareEventOrderOlder> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrderOlder::getOrderId,orderId);
        List<CareEventOrderOlder> ceoos  = this.careEventOrderOlderService.list(queryWrapper);
        if (CollUtil.isNotEmpty(ceoos)){
            ceoos.forEach(item->{
                OrderOlderVO orderOlderVO = new OrderOlderVO();
                BeanUtil.copyProperties(item,orderOlderVO);
                orderOlderVO.setStatusDesc(OrderOlderStatusEnum.getCodeToName(orderOlderVO.getStatus()));
                orderOlderVOList.add(orderOlderVO);
            });

        }
        return orderOlderVOList;
    }

    /**
     *  查询工单的相关联系人
     * @param orderId
     * @return
     */
    private List<OrderContactorVO> queryContactList(Long orderId){
        List<OrderContactorVO> result = Lists.newArrayList();
        QueryWrapper<CareEventOrderContactStatus> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrderContactStatus::getOrderId,orderId).orderByAsc(CareEventOrderContactStatus::getContactLevel);
        List<CareEventOrderContactStatus> ceocs = this.careEventOrderContactStatusService.list(queryWrapper);
        if (CollUtil.isNotEmpty(ceocs)){
            ceocs.forEach(item -> {
                OrderContactorVO contactorVO = new OrderContactorVO();
                BeanUtil.copyProperties(item,contactorVO);
                contactorVO.setStatusDesc(ContactorStatusEnum.getCodeToName(item.getStatus()));
                if (contactorVO.getContactName() == null){
                    contactorVO.setContactName("");
                }
                result.add(contactorVO);
            });
        }
        return  result;
    }

    /**
     * 查询事件处理历史列表
     * @param orderId
     * @return
     */
    private  List<OrderHandleHisVO> queryOrderHandleHis(Long orderId){
        List<OrderHandleHisVO> orderHandleHisVOList = Lists.newArrayList();
        QueryWrapper<CareEventOrderHandleHis> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("distinct org_id,station_id,house_id,order_id,log_type,order_type,log_object_id,log_object_name,log_result," +
                "op_user_role,op_user_id,op_user_name,create_time,relation_type_desc,call_result_id").lambda().eq(CareEventOrderHandleHis::getOrderId,orderId)
                .orderByDesc(CareEventOrderHandleHis::getCreateTime);
        List<CareEventOrderHandleHis> datas = this.careEventOrderHandleHisService.list(queryWrapper);
        if (CollUtil.isNotEmpty(datas)){
            datas.forEach(item ->{
                OrderHandleHisVO hisVO = new OrderHandleHisVO();
                BeanUtil.copyProperties(item,hisVO);
                if (hisVO.getLogObjectName() == null){
                    hisVO.setLogObjectName("");
                }
                orderHandleHisVOList.add(hisVO);
            });
        }
        return orderHandleHisVOList;
    }

    /**
     *  老人 SOS呼叫
     * @param member
     * @param file
     * @return
     */
    public boolean sos(MemberInfoVO member,  MultipartFile file){

//        EventOrderVO orderVO = getDoingEvent(member.getHouseId());
//        if (orderVO!=null){
//            throw new BDException("有进行中事件，请勿重复呼叫");
//        }
//
//        Date now  = DateUtil.date();
//        //1. 保存文件
//        String url = fileUploadService.upload(file,"sos");
//        //2. 查询房屋
//        CareHouse house = this.careHouseService.getById(member.getHouseId());
//        //2. 生成工单
//        CareEventOrder order = new CareEventOrder();
//        order.setOrgId(house.getOrgId());
//        order.setOrgName(this.careOrganizationService.getById(house.getOrgId()).getName());
//        order.setStationId(house.getStationId());
//        order.setStationName(this.careStationService.getById(house.getStationId()).getName());
//        order.setHouseId(house.getId());
//        order.setHouseName(house.getName());
//        order.setTitle(house.getAddr()+" "+member.getName()+" 主动呼叫");
//        order.setOlderId(member.getId());
//        order.setOlderName(member.getName());
//        order.setVoiceUrl(url);
//        order.setOrderType(OrderTypeEnum.ZHU_DONG_HU_JIAO.getValue());
//        order.setStatus(OrderStatusEnum.TODO.getValue());
//        order.setCreateTime(now);
//        order.setModifyTime(now);
//        this.careEventOrderService.saveOrder(order);
        return true;
    }

    /**
     *  老人 SOS呼叫
     * @param member
     * @return
     */
    public boolean sos(MemberInfoVO member){

//        EventOrderVO orderVO = getDoingEvent(member.getHouseId());
//        if (orderVO!=null){
//            throw new BDException("有进行中事件，请勿重复呼叫");
//        }
//        Date now  = DateUtil.date();
//        //2. 查询房屋
//        CareHouse house = this.careHouseService.getById(member.getHouseId());
//        //2. 生成工单
//        CareEventOrder order = new CareEventOrder();
//        order.setOrgId(house.getOrgId());
//        order.setOrgName(this.careOrganizationService.getById(house.getOrgId()).getName());
//        order.setStationId(house.getStationId());
//        order.setStationName(this.careStationService.getById(house.getStationId()).getName());
//        order.setHouseId(house.getId());
//        order.setHouseName(house.getName());
//        order.setTitle(house.getAddr()+" "+member.getName()+" 主动呼叫");
//        order.setOlderId(member.getId());
//        order.setOlderName(member.getName());
//        order.setOrderType(OrderTypeEnum.ZHU_DONG_HU_JIAO.getValue());
//        order.setStatus(OrderStatusEnum.TODO.getValue());
//        order.setCreateTime(now);
//        order.setModifyTime(now);
//        this.careEventOrderService.saveOrder(order);
        return true;
    }

    /**
     * 获取3D监控页面URL
     * @return
     */
    public String getRealTimeMonitorUrl(Long devId){
        Map<String,String> map =  this.careDeviceService.get3dUrl(devId);
        return  map.get("toWX");
    }
}
