package com.care.common.service.impl;

import cn.hutool.core.util.IdUtil;
import com.care.client.config.PlatformOfficialConfig;
import com.care.client.config.WxConfig;
import com.care.client.vo.WxAccessTokenVO;
import com.care.common.cache.RedisKeyConstant;
import com.care.common.cache.RedisUtil;
import com.care.common.service.WxQrCodeService;
import com.care.common.util.WxQrcodeUtil;
import com.care.common.util.WxTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.UUID;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

@Service
@Slf4j
public class WxQrCodeServiceImpl implements WxQrCodeService {

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private PlatformOfficialConfig platformOfficialConfig;

    private Lock lock = new ReentrantLock();

    public String genWxQrcodeUrl(String contactSplicingStr) {
        WxAccessTokenVO wxAccessTokenVO = processOfficialToken();
        WxConfig wxConfig = platformOfficialConfig.officialConfig();
        if (StringUtils.isBlank(wxConfig.getAppId()) || StringUtils.isBlank(wxConfig.getSecret())) {
            log.error("缺少必要参数");
            return null;
        }
        try {
            Integer nextId = NumberUtils.toInt(String.valueOf(System.currentTimeMillis()).substring(4));
            String qrcodeKey = RedisKeyConstant.YUANSERVICE_QRCODE + ":" + nextId;
            if (redisUtil.hasKey(qrcodeKey)) {
                log.error("不能重复产生二维码");
                return null;
            } else {
                redisUtil.set(qrcodeKey, contactSplicingStr,172800);
            }
            String tempTicketUrl = WxQrcodeUtil.createTempTicket(wxAccessTokenVO.getAccessToken(), "172800", nextId);
            if ("40001".equalsIgnoreCase(tempTicketUrl)) {
                redisUtil.del(RedisKeyConstant.YUANSERVICE_ACCESSTOKEN + "-official");
                tempTicketUrl = null;
            }
            log.info("产生二维码: [" + contactSplicingStr + "], URL: " + tempTicketUrl);
            return StringUtils.isNotBlank(tempTicketUrl) ? tempTicketUrl : null;
        } catch (Exception e) {
            log.error("获取二维码出现错误", e);
        }
        return null;
    }

    public WxAccessTokenVO processOfficialToken() {
        WxAccessTokenVO wxAccessTokenVO = new WxAccessTokenVO();
        WxConfig wxConfig = platformOfficialConfig.officialConfig();
        if (StringUtils.isBlank(wxConfig.getAppId()) || StringUtils.isBlank(wxConfig.getSecret())) {
            log.error("缺少必要参数");
            return null;
        }
        try {
            //得到对象锁
            lock.lock();
            if (redisUtil.hasKey(RedisKeyConstant.YUANSERVICE_ACCESSTOKEN + "-official")) {
                wxAccessTokenVO.setAccessToken(ObjectUtils.toString(redisUtil.get(RedisKeyConstant.YUANSERVICE_ACCESSTOKEN + "-official")));
            } else {
                wxAccessTokenVO = WxTemplateUtil.getAccessToken(wxConfig.getAppId(), wxConfig.getSecret());
                if (wxAccessTokenVO == null || StringUtils.isBlank(wxAccessTokenVO.getAccessToken())) {
                    log.error("未获取到AccessToekn");
                    return null;
                }
                redisUtil.set(RedisKeyConstant.YUANSERVICE_ACCESSTOKEN + "-official", wxAccessTokenVO.getAccessToken(),60 * 100);
            }
            return wxAccessTokenVO;
        } catch (Exception e) {
            log.error("获取AccessToken出现错误", e);
        } finally {
            //释放锁
            lock.unlock();
        }
        return null;
    }

}
